# from django_cradmin.viewhelpers.listbuilder import base
#
#
# class Link(base.ItemValueRenderer):
#     """
#     Renders a value item as a link.
#     """
#     def get_url(self):
#         """
#         Get the URL the link should go to.
#
#         Defaults to ``value.get_absolute_url``, which means that this
#         works out of the box with any Django model object with
#         the ``get_absolute_url`` method defined.
#         """
#         return self.value.get_absolute_url()
